module Logistics
  module Core
    class AdditionalDocumentsController < ApplicationController
      before_action :set_service

      def get_lookup_values
        values = @service.get_lookup_values(params[:entity])
        response = Mks::Common::MethodResponse.new(true, nil, values, nil, nil)
        render json: response
      end

      def get_document_types
        documents = @service.get_additional_documents(params[:entity], params[:rec_id])
        data = ActiveModelSerializers::SerializableResource.new(documents).as_json
        response = Mks::Common::MethodResponse.new(true, nil, data, nil, nil)
        render json: response
      end

      def bulk_save
        success, error = @service.bulk_save(additional_document_params[:payload])
        response = Mks::Common::MethodResponse.new(success, nil, nil, [error], nil)
        render json: response
      end

      def bulk_update
        success, error = @service.bulk_update(additional_document_params[:payload])
        response = Mks::Common::MethodResponse.new(success, nil, nil, [error], nil)
        render json: response
      end

      def bulk_delete
        success, error = @service.bulk_delete(params[:ids])
        response = Mks::Common::MethodResponse.new(success, nil, nil, [error], nil)
        render json: response
      end

      private

      def set_service
        @service = AdditionalDocumentService.new
      end

      def additional_document_params
        params.permit(:payload => [:id, :documentable_id, :documentable_type, :document_type_id, :original, :copy,
                                   :certified, :is_endorsed, :is_mandatory, :transaction_type_id])
      end

    end
  end
end