module Logistics
  module Core
    class ContentTypesController < ApplicationController
      before_action :set_content_type, only: [:update]

      # GET /content_types
      # GET /content_types.json
      def index
        @content_types = ContentType.all
        response = Mks::Common::MethodResponse.new(true, nil, @content_types, nil, nil)
        render json: response
      end

      def lookup
        @content_types = ContentType.order(:name).select("id, name")
        response =Mks::Common::MethodResponse.new(true, nil, @content_types, nil, nil)
        render json: response
      end

      # POST /content_types
      # POST /content_types.json
      def create
        @content_type = ContentType.new(content_type_params)
        if @content_type.valid?
          if @content_type.save
            response = Mks::Common::MethodResponse.new(true, "Content Type data Saved Successfully!", @content_type, nil, nil)
          end
        else
          errors = Mks::Common::Util.error_messages @content_type
          response = Mks::Common::MethodResponse.new(false, nil, nil, errors, nil)
        end
        render json: response
      end

      # PATCH/PUT /content_types/1
      # PATCH/PUT /content_types/1.json
      def update
        @old_content_type = set_content_type
        @content_type = ContentType.new(content_type_params)
        if @content_type.valid?
          if @old_content_type.update(content_type_params)
            response = Mks::Common::MethodResponse.new(true, "Content Type data Updated Successfully!", nil, nil)
          end
        else
          errors = Mks::Common::Util.error_messages @content_type
          response = Mks::Common::MethodResponse.new(false, nil, nil, errors, nil)
        end
        render json: response
      end

      private

      def set_content_type
        @content_type = ContentType.find(params[:id])
      end

      def content_type_params
        params.require(:content_type).permit(:code, :name)
      end
    end
  end
end
