module Logistics
  module Core
    class UnitsController < ApplicationController
      before_action :set_unit, only: [:update]

      def index
        units = Unit.includes(:unit_type)
        data = ApplicationRecord.as_json(units)
        response = Mks::Common::MethodResponse.new(true, nil, data, nil, nil)
        render json: response
      end

      def create
        @unit = Unit.new(unit_params)
        if @unit.save
          response = Mks::Common::MethodResponse.new(true, 'Unit saved successfully!', @unit, nil, nil)
        else
          errors = Mks::Common::Util.error_messages @unit, 'Unit'
          response = Mks::Common::MethodResponse.new(false, nil, nil, errors, nil)
        end
        render json: response
      end

      def update
        if @unit.update(unit_params)
          response = Mks::Common::MethodResponse.new(true, 'Unit updated successfully!', @unit, nil, nil)
        else
          errors = Mks::Common::Util.error_messages @unit, 'Unit'
          response = Mks::Common::MethodResponse.new(false, nil, nil, errors, nil)
        end
        render json: response
      end

      private

      def set_unit
        @unit = Unit.find(params[:id])
      end

      def unit_params
        params.require(:unit).permit(:abbreviation, :name, :unit_type_id, :rate_to_base)
      end
    end
  end
end
