module Logistics
  module Core
    class Client < ApplicationRecord
      VALID_EMAIL_REGEX = /\A[\w+\-.]+@[a-z\d\-.]+\.[a-z]+\z/i

      validates :name, presence: true
      validates :email, format: { with: VALID_EMAIL_REGEX }
      belongs_to :country
      belongs_to :client_category

      delegate(:name, to: :country, prefix: true, allow_nil: true)
      delegate(:name, to: :client_category, prefix: true, allow_nil: true)

      before_save { email.downcase! }

      def self.build(options = {})
        Client.new(options)
      end
    end
  end
end
