module Logistics
  module Core
    class Route < ApplicationRecord
      belongs_to :from, class_name: 'Logistics::Core::RouteLookup'
      belongs_to :to, class_name: 'Logistics::Core::RouteLookup'
      belongs_to :node, class_name: 'Logistics::Core::RouteLookup'

      has_many :route_rates

      def self.route_type
        %w(InterCityRoute, WithinCityRoute)
      end

      def route_name
        if from && to
          from.name + ' - ' + to.name
        else
          node_name = node ? node.name : nil
          node_name + ' - ' + self.zone
        end
      end

      def self.fetch_all
        result = []
        Route.all.order('route_type').each do |route|
          result.push({id: route.id,
                       from_id: route.from_id,
                       from: route.from ? route.from.name : nil,
                       to_id: route.to_id,
                       to: route.to ? route.to.name : nil,
                       route_name: route.route_name,
                       node_id: route.node_id,
                       node_name: route.node ? route.node.name : nil,
                       zone: route.zone,
                       radius: route.radius,
                       route_type: route.route_type,
                       description: route.description,
                       margin: route.margin
                      })
        end
        return result
      end
    end
  end
end