require 'active_model_serializers'
require 'mks_auth'

module Logistics
  module Core
    class Engine < ::Rails::Engine
      isolate_namespace Logistics::Core

      initializer :append_migrations do |app|
        unless app.root.to_s.match root.to_s
          if app.config.app_code == 'PSH'
            config.paths['db/migrate'].expanded.each do |expanded_path|
              app.config.paths['db/migrate'] << expanded_path
            end
          end
        end
      end

      initializer 'logistics_core.factories', :after => 'factory_girl.set_factory_paths' do
        FactoryGirl.definition_file_paths << File.expand_path('../../../../spec/factories', __FILE__) if defined?(FactoryGirl)
      end

      config.generators do |g|
        g.test_framework :rspec, :fixture => false
        g.fixture_replacement :factory_girl, :dir => 'spec/factories'
        g.assets false
        g.helper false
      end
    end
  end
end
